
%input parameters
x0 = 0.0110;
y0 = 0.9904;
Qn = 5.911;
Qp = 5.389;

V_max = max(VPModel.FullCell.NEI.Vc);
V_min = min(VPModel.FullCell.NEI.Vc);
step = 0.0005;

% Cell initialisation
HC_neg = VPModel.HalfCell.NEI.Graphite.Vc;%update the NE
HC_neg_stoich = VPModel.HalfCell.NEI.Graphite.x;%update the NE
 
HC_pos = VPModel.HalfCell.NEI.NMC532.Vc; %update the PE
HC_pos_stoich = VPModel.HalfCell.NEI.NMC532.y; %update the PE

FC_cell = VPModel.FullCell.NEI.Vc;
FC_Q = VPModel.FullCell.NEI.Q;
FC_DVA = VPModel.FullCell.NEI.DVA;
FC_ICA = VPModel.FullCell.NEI.ICA;

% Voltage Profile reconstruction

c = 0:step:Qn;
xn = x0 + (c./Qn);
yp = y0 - (c./Qp);

Vn = interp1(HC_neg_stoich,HC_neg,xn,'linear');
Vp = interp1(HC_pos_stoich,HC_pos,yp,'linear');
Vc = Vp-Vn;

FullCell.Vc = Vc(Vc>=V_min & Vc<=V_max);
FullCell.Q = c(Vc>=V_min & Vc<=V_max);

FullCell.Vn = Vn(Vc>=V_min & Vc<=V_max);
FullCell.xn = xn(Vc>=V_min & Vc<=V_max);
FullCell.Vp = Vp(Vc>=V_min & Vc<=V_max);
FullCell.yp = yp(Vc>=V_min & Vc<=V_max);
FullCell.SOC = FullCell.Q./max(FullCell.Q); 

x100 = max(xn(Vc>=V_min & Vc<=V_max));
y100 = min(yp(Vc>=V_min & Vc<=V_max));

% Incremental capacity and Differential Voltage Analysis

order = 1;
i = find(FullCell.SOC >= 0.02,1);
IS_EVEN = ~bitget(abs(i),1);
if IS_EVEN==1
    framelen = i+1;
else
    framelen = i+2;
end

Vcf = sgolayfilt(FullCell.Vc,order,framelen);
dVc = gradient(Vcf);
dQ = gradient(FullCell.Q);

FullCell.ICA = dQ./dVc;        
FullCell.DVA = dVc./dQ;


% Fitting goodness

Vc_interp = interp1(FC_Q,FC_cell,FullCell.Q,'linear','extrap');
Diff_Vc = (Vc_interp-FullCell.Vc).^2;%IC overall
RMSE_Vc = sqrt(mean(Diff_Vc));

IC_interp = interp1(FC_cell,FC_ICA,FullCell.Vc,'linear','extrap');
Diff_IC = (IC_interp-FullCell.ICA).^2;%IC overall
RMSE_IC = sqrt(mean(Diff_IC));

DV_interp = interp1(FC_Q,FC_DVA,FullCell.Q,'linear','extrap');
Diff_DV = (DV_interp-FullCell.DVA).^2;%DV overall
RMSE_DV = sqrt(mean(Diff_DV));%DV overall

Vhatmin = min(FullCell.Vc);
Vmin_err = Vhatmin-V_min;

Vhatmax = max(FullCell.Vc);
Vmax_err = Vhatmax-V_max;

err_Q = (max(FC_Q)-min(FC_Q))-((max(FullCell.Q)-min(FullCell.Q)));


%% Plotting
close all

figure(1)
hold on
plot(HC_neg_stoich,HC_neg,'r-','linewidth',1.5)
plot(FullCell.xn,FullCell.Vn,'k-.','linewidth',1.5)
xlabel('stoichiometry'),ylabel('Potential, V'),ylim([0 1.4])
title('NE Half-cell profile'),legend('Vn','VHCNE lim','VHCNE'),grid on,box on
movegui('center');

figure(2)
hold on
plot(HC_pos_stoich,HC_pos,'r-','linewidth',1.5)
plot(FullCell.yp,FullCell.Vp,'k-.','linewidth',1.5)
xlabel('stoichiometry'),ylabel('Potential, V')
title('PE Half-cell profile'),legend('Vp','VHCPE lim','VHCPE'),grid on,box on
movegui('northwest');

figure(3)
hold on
plot(FullCell.Q,FullCell.Vc,'r-','linewidth',1.5)
plot(FC_Q,FC_cell,'k-','linewidth',1.5)
xlabel('Capacity,mAh'),ylabel('Voltage, V')
yyaxis right,ylabel('error, V'),ylim([0 0.009]),set(gca,'YColor','k')
plot(FullCell.Q,Diff_Vc,'b-.','linewidth',1)
title(sprintf('Cell OCV : RMSE %.4f',RMSE_Vc)),grid on,box on
movegui('northeast');

figure(4)
hold on
plot(FullCell.Vc,FullCell.ICA,'r-','linewidth',1.5)
plot(FC_cell,FC_ICA,'k-','linewidth',1.5)
xlabel('Potential,V'),ylabel('ICA, mAh/V'),xlim([3.2 4.2])
yyaxis right,ylabel('error'),ylim([0 0.12]),set(gca,'YColor','k')
%plot(FullCell.Vc,Diff_IC,'b-.','linewidth',1)
title(sprintf('ICA : RMSE %.4f',RMSE_IC)),grid on,box on
movegui('southeast');

figure(5)
hold on
plot(FullCell.Q,FullCell.DVA,'r-','linewidth',1.5)
plot(FC_Q,FC_DVA,'k-','linewidth',1.5)
xlabel('Capacity, mAh'),ylabel('DVA, V/mAh'),ylim([0 0.5])
yyaxis right,ylabel('error'),ylim([0 0.05]),set(gca,'YColor','k')
plot(FullCell.Q,Diff_DV,'b-.','linewidth',1)
title(sprintf('DVA : RMSE %.4f',RMSE_DV)),grid on,box on
movegui('southwest');
